/*
OBIETTIVI: usare stack e ricorsione

Una stringa  palindroma se letta nei due versi risulta uguale.
Esempi di stringhe palindrome:
ANNa
RaDAr
OTto
MADAmIMaDAM
EVe

Se si ignorano i caratteri diversi dalle lettere, sono palindrome:
Madam, I'm Adam.         
Able was I 'ere I saw Elba.    
A man. A plan. A canal. Panama!

ESERCIZIO:
Scrivere un metodo che usa la ricorsione per verificare se una stringa  palindroma


RICORDA:
substring(int beginIndex [, int endIndex])
restituisce una nuova stringa che e' una sottostringa della stringa corrente.
La sottostringa inizia con il carattere in posizione beginIndex e si estende
fino alla fine della stringa oppure fino alla posizione endIndex-1 se
il secondo parametro e' specificato.
*/



public class Palindrome { 


/*********************************************************************************
* VERSIONE 1 - ricorsione
* metodo di base semplificato valido per stringhe composte
* da sole lettere tutte maiuscole/minuscole  ("ANNA")
**********************************************************************************/
public static boolean isPalindrome (String s) {
         if (s.length() <= 1)
             return true;
         if (s.charAt (0) != s.charAt (s.length() - 1))
             return false;
         return isPalindrome (s.substring (1, s.length() - 1)); 
    } // method isPalindrome


/*********************************************************************************
* VERSIONE 2 - ricorsione
* Estensione del metodo valida senza distinzione tra lettere maiuscole e minuscole,
* ignorando caratteri diversi dalle lettere.
* Ad esempio, sono palindrome:
* Madam, I'm Adam.         
* Able was I 'ere I saw Elba.    
* A man. A plan. A canal. Panama!
**********************************************************************************/

    /** METODI AUSILIARI **/

    public static boolean islowerletter(char c){
    	return (c >='a' && c <='z');
    }

    public static boolean isupperletter(char c){
    	return (c >='A' && c <='Z');
    }

    public static boolean isletter(char c){
    	return islowerletter(c) || isupperletter(c);
    }


    public static boolean isPalindrome2 (String s) 
    {                     
       String letters = new String(); //copia di s privata dei caratteri diversi dalle lettere

       for (int i = 0; i < s.length(); i++){
    	   char c=s.charAt(i);
	   if (isletter(c)) letters += c;
       }//end-for	
       if (letters.length() == 0) return true;
       letters = letters.toUpperCase();        
       return isPalindrome (letters);
    } // method isPalindrome2


/*********************************************************************************
* VERSIONE 3 - iterazione
*********************************************************************************/

    public static char toupperletter(char c){
    	if (islowerletter(c)) return (char)('A'+(c-'a'));
    	else return c;
    }
    
   public static boolean isPalindrome3 (String s) {
   if (s.length() <= 1) return true;

   int i = 0, j = s.length() - 1;
   while (i<j) {
     char c1 = s.charAt(i);
     while (!isletter(c1) && i<j) i++; //ignora caratteri non lettera
     char c2 = s.charAt(j);
     while (!isletter(c2) && i<j) j--; //ignora caratteri non lettera
     if (i==j) return true;
     else if (toupperletter(c1) != toupperletter(c2)) return false;
     i++; j--;
   } 
   return true;
} // method isPalindrome3


/*********************************************************************************
 * VERSIONE 4 - stack
 *********************************************************************************/
   
   public static boolean isPalindrome4 (String s) {
	   if (s.length() <= 1) return true;

	   MyStack<Character> stk= new ArrayStack<Character>();

	   int i = 0, len = s.length();
	   String letters = new String();
	   while (i < len) {
		   char c1 = s.charAt(i++);
		   if (isletter(c1)) {
			   c1 = toupperletter(c1);
			   letters += c1;
			   stk.push(c1);
		   }
	   }
	   i=0;
	   boolean ispalindrome = true;
	   len = letters.length();
	   while (ispalindrome && i < len){
		   if (letters.charAt(i++) != stk.pop()) ispalindrome = false;
	   }
	   return ispalindrome;

	} // method isPalindrome4   
   
   
/************************************************
 * SI PUO' MIGLIORARE il metodo isPalindrome4()? 
 * banale... si pu verificare la corrispondenza solo su met stringa ()   
 ************************************************/
  
   public static boolean isPalindrome5 (String s) {
	   if (s.length() <= 1) return true;

	   MyStack<Character> stk= new ArrayStack<Character>();

	   int i = 0, len = s.length();
	   String letters = new String();
	   while (i < len) {
		   char c1 = s.charAt(i++);
		   if (isletter(c1)) {
			   c1 = toupperletter(c1);
			   letters += c1;
			   stk.push(c1);
		   }
	   }   
	   i=0;
	   boolean ispalindrome = true;
	   len = letters.length()/2;  // ()
	   while (ispalindrome && i < len){
		   if (letters.charAt(i++) != stk.pop()) ispalindrome = false;
	   }
	   return ispalindrome;

	} // method isPalindrome5
   
   
   public static void main(String args[]) {
	String s="bAn!n  a.B";
	if (isPalindrome5(s)) 
	   System.out.println("\"" + s + "\" e' palindroma");
	else
	   System.out.println("\"" + s + "\" non e' palindroma");
   }   
   
}//end-class
